import os
from openai import OpenAI

client = OpenAI(
    api_key=os.getenv('OPEN_API_KEY'),
)

prompt = """
Generate synthetic data for promotional offers by detailing each offer's ID, title, description, category, discount, validity dates, and target audience demographics, and format it consistently in JSON for use in recommendation systems.
"""

chat_completion = client.chat.completions.create(
    messages=[
        {
            "role": "user",
            "content": prompt,
        }
    ],
    model="gpt-3.5-turbo",
)



import json
import httpx
import requests
from types import FunctionType
from ..utils.logger import logger
from ..utils.welcome import welcome
from typing import Dict, Optional


class NsSocket(object):
    def __init__(
            self,
            grpc_endpoint: str = "api.nstream.ai:50031",
            api_server_url: str = "https://api.nstream.ai",
            headers: dict = {},
            dashboard_server: str = "https://api.nstream.ai/graphql") -> None:
        self.grpc_endpoint = grpc_endpoint
        self.http_client = httpx.Client()
        self.api_server = api_server_url
        self.dashboard_server = dashboard_server
        self.status = False
        self.headers = headers
        logger.info("NsSocket initialized with API server: {0}".format(api_server_url))

    def call_grpc_endpoint(self, method: FunctionType):
        self.status = True
        logger.info("gRPC endpoint called")
        return self.status

    def call_rest_endpoint(self,
                           payload: Optional[Dict] = {},
                           params: Optional[Dict] = None,
                           method: str = "GET",
                           route: str = "/"):
        endpoint = "{0}/{1}".format(self.api_server, route)
        logger.info("Calling REST endpoint: {0} with method: {1}".format(endpoint, method))
        if method == "GET":
            response = self.http_client.get(headers=self.headers, url=endpoint, params=params)
        elif method == "POST":
            response = self.http_client.post(headers=self.headers, url=endpoint, json=payload)
        elif method == "PUT":
            response = self.http_client.put(headers=self.headers, url=endpoint, data=payload)
        elif method == "DELETE":
            response = requests.request("DELETE", url=endpoint, headers=self.headers, data=payload)
        else:
            response = httpx.Response(status_code=500, content=None, text=json.dumps({"status": "failed", "reason": "No method allowed"}))
            logger.error("Invalid HTTP method: {0}".format(method))
        return response

    def terminate_client(self):
        self.http_client.close()
        logger.info("HTTP client terminated")


class NsInit(object):
    def __init__(self, api_key="", username="", password="") -> None:
        welcome()
        self.api_key = api_key
        self.username = username
        self.password = password
        self.socket = NsSocket()
        self.headers = {}
        self.params = {}
        logger.info("NsInit initialized with API key: {0}".format(api_key))

    def connect(self) -> NsSocket:
        try:
            result = self.socket.call_rest_endpoint(method="POST", route="sign-in", payload={"email": self.username, "password": self.password})
            print(result)
            oauth_token = result.json().get("access_token")
            if oauth_token:
                self.headers = {"Authorization": "Bearer {0}".format(oauth_token), 'Content-Type': 'application/json'}
                self.socket = NsSocket(headers=self.headers)
                logger.info("Authorization Token Received")
            else:
                logger.error("Failed to retrieve authorization token")
                raise Exception("Failed to authenticate")

            result = self.socket.call_rest_endpoint(route="validate-api-key/{}".format(self.api_key))
            result = result.json()
            api_secret = result.get("api_secret")
            if api_secret:
                logger.info("API Key approved")
                return self.socket
            else:
                logger.error("API Key {0} Invalid, please check if API Key exists".format(self.api_key))
                raise Exception("API Key does not exist")
        except Exception as e:
            logger.exception("Unknown Exception Occurred - {0}".format(e))
            raise Exception("Unknown Exception Occurred - {0}".format(e))



import logging

# ANSI escape codes for coloring
class LogColors:
    RESET = "\033[0m"
    DEBUG = "\033[34m"  # Blue
    INFO = "\033[32m"   # Green
    WARNING = "\033[33m" # Yellow
    ERROR = "\033[31m"   # Red
    CRITICAL = "\033[35m" # Magenta

class CustomFormatter(logging.Formatter):
    """Logging Formatter to add colors and count warning / errors"""

    format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s (" \
             "%(filename)s:%(lineno)d)"

    FORMATS = {
        logging.DEBUG: LogColors.DEBUG + format + LogColors.RESET,
        logging.INFO: LogColors.INFO + format + LogColors.RESET,
        logging.WARNING: LogColors.WARNING + format + LogColors.RESET,
        logging.ERROR: LogColors.ERROR + format + LogColors.RESET,
        logging.CRITICAL: LogColors.CRITICAL + format + LogColors.RESET
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt, datefmt="%Y-%m-%d %H:%M:%S")
        return formatter.format(record)

def get_logger(name):
    # Create a custom logger
    logger = logging.getLogger(name)

    # Set the log level
    logger.setLevel(logging.DEBUG)  # Set to the lowest level to capture all messages

    # Create handlers
    c_handler = logging.StreamHandler()
    c_handler.setLevel(logging.DEBUG)

    # Create formatters and add it to handlers
    c_format = CustomFormatter()
    c_handler.setFormatter(c_format)

    # Add handlers to the logger
    logger.addHandler(c_handler)

    return logger

# Define the global logger variable
logger = get_logger("nstreamai")

# Example usage:
if __name__ == "__main__":
    logger.debug('This is a debug message')
    logger.info('This is an info message')
    logger.warning('This is a warning message')
    logger.error('This is an error message')
    logger.critical('This is a critical message')



from logger import get_logger
from output_data import chat_completion
from run import send_graphql_request, generate_synthetic_data, random_json, random_string
from template import create_node_detail_mutation, update_node_detail_mutation, create_data_detail_mutation, create_token_detail_mutation, update_token_detail_mutation, create_io_throughput_mutation, update_io_throughput_mutation, create_inference_latency_mutation, update_inference_latency_mutation, create_node_message_mutation
from variables import generate_synthetic_data, send_graphql_request
from welcome import welcome

__all__ = ['get_logger', 
           'chat_completion', 
           'send_graphql_request', 
           'generate_synthetic_data', 
           'random_json',
           'random_string',
           'create_node_detail_mutation',
           'update_node_detail_mutation',
           'create_data_detail_mutation',
           'create_token_detail_mutation',
           'update_token_detail_mutation',
           'create_io_throughput_mutation',
           'update_io_throughput_mutation',
           'create_inference_latency_mutation',
           'update_inference_latency_mutation',
           'create_node_message_mutation',
           'welcome'
           ]


import json


def create_node_detail_mutation(name, context_size, prompt_size, prompt, context,
                                total_data_processed, model_name):
    return f"""
    mutation {{
      createNodeDetail(
        name: "{name}"
        contextSize: {context_size}
        promptSize: {prompt_size}
        totalDataProcessed: {total_data_processed}
        neuron: "{model_name}"
        prompt: "{prompt}"
        context: "{context}"
      ) {{
        ... on NodeType {{
          id
          name
          orgId
          userId
          contextSize
          promptSize
          totalDataProcessed
          activeSince
          neuron
        }}
        ... on ErrorType {{
          message
        }}
      }}
    }}
    """


def update_node_detail_mutation(id, name, context_size, prompt_size, prompt, context):
    return f"""
    mutation {{
      updateNodeDetail(
        id: "{id}"
        name: "{name}"
        contextSize: {context_size}
        promptSize: {prompt_size}
        prompt: "{prompt}"
        context: "{context}"
      ) {{
        ... on NodeType {{
          id
          name
          orgId
          userId
          contextSize
          promptSize
        }}
        ... on ErrorType {{
          message
        }}
      }}
    }}
    """


def create_data_detail_mutation(type, node_id, avg_throughput, link_metadata,
                                role):
    link_metadata_str = json.dumps(link_metadata).replace('"', '\\"')
    return f"""
    mutation {{
      createDataDetail(
        type: "{type}"
        nodeId: "{node_id}"
        avgThroughPut: {avg_throughput}
        linkMetadata: "{link_metadata_str}"
        role: "{role}"
      ) {{
        ... on DataType {{
          id
          type
          orgId
          userId
          nodeId
          avgThroughPut
          linkMetadata
        }}
        ... on ErrorType {{
          message
        }}
      }}
    }}
    """


def create_token_detail_mutation(tokens, node_id):
    return f"""
    mutation {{
      createTokenDetail(
        tokens: {tokens}
        nodeId: {node_id}
      ) {{
        ... on TokenDetailType {{
          id
          orgId
          userId
          modelName
          tokens
          nodeId
        }}
        ... on ErrorType {{
          message
        }}
      }}
    }}
    """


def update_token_detail_mutation(id, model_name, tokens):
    return f"""
    mutation {{
      updateTokenDetail(
        id: "{id}"
        modelName: "{model_name}"
        tokens: {tokens}
      ) {{
        ... on TokenDetailType {{
          id
          orgId
          userId
          modelName
          tokens
          nodeId
        }}
        ... on ErrorType {{
          message
        }}
      }}
    }}
    """


def create_io_throughput_mutation(node_id, input_throughput,
                                  output_throughput):
    return f"""
    mutation {{
      createIoThroughput(
        nodeId: {node_id}
        inputThroughput: {input_throughput}
        outputThroughput: {output_throughput}
      ) {{
        ... on IOThroughputType {{
          id
          nodeId
          orgId
          userId
          inputThroughput
          outputThroughput
        }}
        ... on ErrorType {{
          message
        }}
      }}
    }}
    """


def update_io_throughput_mutation(id, input_throughput, output_throughput):
    return f"""
    mutation {{
      updateIoThroughput(
        id: "{id}"
        inputThroughput: {input_throughput}
        outputThroughput: {output_throughput}
      ) {{
        ... on IOThroughputType {{
          id
          nodeId
          orgId
          userId
          inputThroughput
          outputThroughput
        }}
        ... on ErrorType {{
          message
        }}
      }}
    }}
    """


def create_inference_latency_mutation(node_id, llm_inference_speed,
                                      context_retrieval_speed,
                                      total_node_inference_speed):
    return f"""
    mutation {{
      createInferenceLatency(
        nodeId: {node_id}
        llmInferenceSpeed: {llm_inference_speed}
        contextRetrievalSpeed: {context_retrieval_speed}
        totalNodeInferenceSpeed: {total_node_inference_speed}
      ) {{
        ... on InferenceLatencyType {{
          id
          nodeId
          orgId
          userId
          llmInferenceSpeed
          contextRetrievalSpeed
          totalNodeInferenceSpeed
        }}
        ... on ErrorType {{
          message
        }}
      }}
    }}
    """


def update_inference_latency_mutation(id, llm_inference_speed,
                                      context_retrieval_speed,
                                      total_node_inference_speed):
    return f"""
    mutation {{
      updateInferenceLatency(
        id: "{id}"
        llmInferenceSpeed: {llm_inference_speed}
        contextRetrievalSpeed: {context_retrieval_speed}
        totalNodeInferenceSpeed: {total_node_inference_speed}
      ) {{
        ... on InferenceLatencyType {{
          id
          nodeId
          orgId
          userId
          llmInferenceSpeed
          contextRetrievalSpeed
          totalNodeInferenceSpeed
        }}
        ... on ErrorType {{
          message
        }}
      }}
    }}
    """

def create_node_message_mutation(message, node_id, key):
    """
    Constructs a GraphQL mutation string for creating a node message.

    :param message: str - The message content in JSON format.
    :param node_id: int - The ID of the node.
    :param key: int - A unique key associated with the message.
    :return: str - GraphQL mutation string.
    """
    # Convert the dictionary to a JSON string
    message_json = json.dumps(message)

    # Manually escape the double quotes for GraphQL
    message_for_graphql = message_json.replace('"', '\\"')
    
    return f"""
    mutation {{
      createNodeMessage(
        message: "{message_for_graphql}"
        nodeId: {node_id}
        key: {key}
      ) {{
        ... on NodeMessage {{
          id
          createdOn
          key
          message
        }}
        ... on ErrorTypeMessage {{
          __typename
          messages
        }}
      }}
    }}
    """




import json
import random
import string
import requests
from template import (create_node_detail_mutation,
                      create_token_detail_mutation,
                      create_io_throughput_mutation,
                      create_inference_latency_mutation,
                      create_data_detail_mutation)


def random_string(length=10):
    return ''.join(random.choices(string.ascii_letters, k=length))


def random_json():
    return json.dumps({
        random_string(5): random_string(5)
        for _ in range(random.randint(1, 1))
    })


def generate_synthetic_data():
    model_names = ["Mistral-7B", "LLAMA2-7B"]  # List of model names
    model_name = random.choice(model_names)
    tokens = random.randint(50, 500)
    input_throughput = random.uniform(50, 1000)
    output_throughput = random.uniform(50, 1000)
    llm_inference_speed = random.uniform(10, 5000)
    context_retrieval_speed = random.uniform(10, 5000)
    total_node_inference_speed = llm_inference_speed + context_retrieval_speed + random.uniform(
        10, 100)

    return {
        "model_name": model_name,
        "tokens": tokens,
        "input_throughput": input_throughput,
        "output_throughput": output_throughput,
        "llm_inference_speed": llm_inference_speed,
        "context_retrieval_speed": context_retrieval_speed,
        "total_node_inference_speed": total_node_inference_speed
    }


def send_graphql_request(url, headers, mutation):
    payload = json.dumps({"query": mutation})
    response = requests.post(url, headers=headers, data=payload)
    return response


url = "http://0.0.0.0:8000/graphql"
headers = {
    'Content-Type':
    'application/json',
    'Authorization':
    'Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiJwaXl1c2hAbnN0cmVhbS5haSIsImV4cCI6MTcxNzc0MTcwNn0.DOvgE2BkG9-tqme2DjfH171Kb4vKH-Yb6v2TINhnCqU'
}

node_id_list = []

#  Node 1
node_name = f"Node 1"
prompt_size = random.uniform(20, 80)
context_size = random.uniform(70, 120)
total_data_processed = random.randint(5, 20)
model_name = "Mistral-7B"
mutation = create_node_detail_mutation(node_name, context_size, prompt_size,
                                       total_data_processed, model_name)
response = send_graphql_request(url, headers, mutation)
print(response.text)
node_id = json.loads(
    response.text).get("data").get("createNodeDetail").get("id")
node_id_list.append(node_id)

#  datasource - prompt
avg_throughput = random.uniform(300, 500)
link_metadata = {"S3": "value"}
data_input = "datasource"
role = "prompt"
mutation = create_data_detail_mutation(data_input, node_id, avg_throughput,
                                       link_metadata, role)
response = send_graphql_request(url, headers, mutation)

#  datasource - context
avg_throughput = random.uniform(300, 500)
link_metadata = {"S3": "value"}
data_input = "datasource"
role = "context"
mutation = create_data_detail_mutation(data_input, node_id, avg_throughput,
                                       link_metadata, role)
response = send_graphql_request(url, headers, mutation)

#  Node 2
node_name = f"Node 2"
prompt_size = random.uniform(20, 80)
context_size = random.uniform(70, 120)
total_data_processed = random.randint(5, 20)
model_name = "LLAMA2-7B"
mutation = create_node_detail_mutation(node_name, context_size, prompt_size,
                                       total_data_processed, model_name)
response = send_graphql_request(url, headers, mutation)
print(response.text)
node_id = json.loads(
    response.text).get("data").get("createNodeDetail").get("id")
node_id_list.append(node_id)

#  datasource - prompt
avg_throughput = random.uniform(300, 500)
link_metadata = {"S3": "value"}
data_input = "datasource"
role = "prompt"
mutation = create_data_detail_mutation(data_input, node_id, avg_throughput,
                                       link_metadata, role)
response = send_graphql_request(url, headers, mutation)

#  datasource - context
avg_throughput = random.uniform(300, 500)
link_metadata = {"S3": "value"}
data_input = "datasource"
role = "context"
mutation = create_data_detail_mutation(data_input, node_id, avg_throughput,
                                       link_metadata, role)
response = send_graphql_request(url, headers, mutation)

#  datasink
avg_throughput = random.uniform(300, 500)
link_metadata = {"S3": "value"}
data_input = "datasink"
role = "output"
mutation = create_data_detail_mutation(data_input, node_id, avg_throughput,
                                       link_metadata, role)
response = send_graphql_request(url, headers, mutation)

try:
    while True:
        # Generate synthetic data
        data = generate_synthetic_data()
        data["node_id"] = random.choice(node_id_list)

        mutation = create_token_detail_mutation(data["tokens"],
                                                data["node_id"])
        response = send_graphql_request(url, headers, mutation)

        mutation = create_io_throughput_mutation(data["node_id"],
                                                 data["input_throughput"],
                                                 data["output_throughput"])
        response = send_graphql_request(url, headers, mutation)

        mutation = create_inference_latency_mutation(
            data["node_id"], data["llm_inference_speed"],
            data["context_retrieval_speed"],
            data["total_node_inference_speed"])
        response = send_graphql_request(url, headers, mutation)

except KeyboardInterrupt:
    print("Loop interrupted by user.")



import pyfiglet
from termcolor import colored


def welcome():
    print(
        colored("_______________________________________________________",
                'light_blue'))
    print("                                                    ")
    pyfiglet.print_figlet(text="Nstream AI ", colors="BLUE")
    print(
        colored("______________________________________________________",
                'light_blue'))
    print("                                                    ")
    print(colored("Copyright © 2024-25", 'dark_grey'))
    print("                                                    ")
    print("                                                    ")
    print("                                                    ")
    print(
        colored("A High Performance Stream Processor powered by Gen AI",
                'red'))
    print("                                                    ")
    print(colored("Dashboard at https://console.nstream.ai", 'light_cyan'))
    print("                                                    ")
    print("                                                    ")
    print("                                                    ")



from ..utils.logger import logger


class NstreamLLM(object):

    def __init__(self) -> None:
        logger.info("Initializing NstreamLLM")
        pass

    @staticmethod
    def feret_v1():
        logger.info("Fetching FERET_V1 model name")
        return "FERET_V1"

    @staticmethod
    def llama2_7b():
        logger.info("Fetching LLAMA2_7B model name")
        return "LLAMA2_7B"

    @staticmethod
    def mistral_7b():
        logger.info("Fetching MISTRAL_7B model name")
        return "MISTRAL_7B"


class NsNeuron(object):

    def __init__(self, llm: str) -> None:
        logger.info(f"Initializing NsNeuron with LLM: {llm}")
        self.llm = llm
        pass



from nsgraph import NsGraph
from nsinit import NsInit, NsSocket
from nsneuron import NsNeuron, NstreamLLM
from nsnode import NsNode, NsLink, NsProvider, NsProviderType, NsDataObject, Nstream

__all__ = ['NsGraph', 'NsInit', 'NsSocket', 'NsNeuron', 'NstreamLLM', 'NsNode', 'NsLink', 'NsProvider', 'NsProviderType', 'NsDataObject', 'Nstream']


import json
import random
import string
import requests


def random_string(length=10):
    return ''.join(random.choices(string.ascii_letters, k=length))


def random_json():
    return json.dumps({
        random_string(5): random_string(5)
        for _ in range(random.randint(1, 1))
    })


def generate_synthetic_data():
    tokens = random.randint(50, 500)
    input_throughput = random.randint(50, 1000)
    output_throughput = random.randint(50, 1000)
    llm_inference_speed = random.randint(10, 5000)
    context_retrieval_speed = random.randint(10, 5000)
    total_node_inference_speed = llm_inference_speed + context_retrieval_speed + random.randint(
        10, 100)

    return {
        "tokens": tokens,
        "input_throughput": input_throughput,
        "output_throughput": output_throughput,
        "llm_inference_speed": llm_inference_speed,
        "context_retrieval_speed": context_retrieval_speed,
        "total_node_inference_speed": total_node_inference_speed
    }


def send_graphql_request(url, headers, mutation):
    payload = json.dumps({"query": mutation})
    response = requests.post(url, headers=headers, data=payload)
    return response



import time
import random
from typing import Dict
from ..core.nsinit import NsSocket
from ..core.nsnode import NsNode, NsLink
from ..utils.variables import generate_synthetic_data, send_graphql_request
from ..utils.template import (
    create_token_detail_mutation, 
    create_io_throughput_mutation, 
    create_inference_latency_mutation,
    create_node_message_mutation
    )
from ..utils.logger import logger
import json
from ..utils.output_data import chat_completion


class NsGraph(object):

    def __init__(self, socket: NsSocket) -> None:
        self.graph = list(dict())
        self.socket = socket
        self.current_node = None
        self.last_node = None
        self.list_node_id = []
        logger.info("NsGraph initialized")
        pass

    def start(self, node: NsNode):
        logger.info(f"Starting graph with node {node.node_name}")
        self.current_node = node
        node.process()
        self.list_node_id.append(node.node_id)
        return self

    def next_node(self, node: NsNode):
        logger.info(f"Moving to next node {node.node_name}")
        self.last_node = self.current_node
        self.current_node = node
        self.graph.append(self.node_to_dict(self.current_node))
        node.process()
        self.list_node_id.append(node.node_id)
        return self

    def end(self, node: NsNode):
        logger.info(f"Ending graph with node {node.node_name}")
        self.last_node = self.current_node
        self.current_node = node
        self.graph.append(self.node_to_dict(self.current_node))
        node.process()
        self.list_node_id.append(node.node_id)
        return self

    def submit(self, sink: NsLink):
        logger.info(f"Submitting graph with sink {sink.provider.NsProviderName}")
        self.graph.append({"sink": sink})
        self.socket.call_grpc_endpoint(method=(lambda x: x))
        sink.process_sink(self.list_node_id[-1])
        return self

    def terminate(self, run_time):
        logger.info("Processing graph")
        if run_time:
            self.run_data_out(run_time=run_time)
            payload = json.dumps([int(i) for i in self.list_node_id])
            # _ = self.socket.call_rest_endpoint(method="DELETE", route="nodes", payload=payload)
        self.socket.call_grpc_endpoint(method=(lambda x: x))
        return self

    @staticmethod
    def node_to_dict(node: NsNode) -> Dict:
        node_dict = dict()
        return node_dict

    def run_data_out(self, run_time) -> None:
        st = time.time()
        while run_time > 0:
            # Generate synthetic data
            data = generate_synthetic_data()
            data["node_id"] = random.choice(self.list_node_id)

            mutation = create_token_detail_mutation(data["tokens"],
                                                    data["node_id"])
            _ = send_graphql_request(self.socket.dashboard_server,
                                     self.socket.headers, mutation)

            mutation = create_io_throughput_mutation(data["node_id"],
                                                     data["input_throughput"],
                                                     data["output_throughput"])
            _ = send_graphql_request(self.socket.dashboard_server,
                                     self.socket.headers, mutation)

            mutation = create_inference_latency_mutation(
                data["node_id"], data["llm_inference_speed"],
                data["context_retrieval_speed"],
                data["total_node_inference_speed"])
            _ = send_graphql_request(self.socket.dashboard_server,
                                     self.socket.headers, mutation)
            
            # Example usage
            message = chat_completion.choices[0].message.content
            key = random.randint(1, 10)
            message = json.loads(message)
            mutation = create_node_message_mutation(message=message, node_id=int(data["node_id"]), key=key)
            result = send_graphql_request(
                self.socket.dashboard_server,
                self.socket.headers,
                mutation
                )
            logger.info(result.json())
            logger.info(f"Pushed Output")

            sleep_time = 2
            time.sleep(sleep_time)
            run_time -= sleep_time
            logger.info(f"Processing nstream, time remaining: {run_time}")
        et = time.time()
        logger.info(f"Completed in {et - st} seconds")



import json
import random
from ..core.nsinit import NsSocket
from typing import Dict, Optional
from ..core.nsneuron import NsNeuron
from ..utils.variables import send_graphql_request
from ..utils.template import create_node_detail_mutation, create_data_detail_mutation
from ..utils.logger import logger

class NsProviderType():
    Sink: str
    Source = str

    def __init__(self) -> None:
        self.Sink = "SINK"
        self.Source = "SOURCE"
        logger.info("NsProviderType initialized")


class NsDataObject():
    NsProviderName: str
    NsProviderMeta: Dict
    NsProviderType: str

    def __init__(self, ns_provider_name, ns_provider_meta,
                 ns_provider_type) -> None:
        self.NsProviderName = ns_provider_name
        self.NsProviderMeta = ns_provider_meta
        self.NsProviderType = ns_provider_type
        logger.info(f"NsDataObject initialized with provider: {ns_provider_name}")
        pass

    def get(self):
        logger.debug(f"Getting data object for provider: {self.NsProviderName}")
        return self


class NsProvider(object):

    def __init__(self, type: str) -> None:
        self.type = type
        logger.info(f"NsProvider initialized with type: {type}")

    def mongodb(self, **kwargs):
        logger.info("Configuring MongoDB provider")
        return NsDataObject(ns_provider_meta=kwargs,
                            ns_provider_name="MONGODB",
                            ns_provider_type=self.type)

    def postgresql(self, **kwargs):
        logger.info("Configuring PostgreSQL provider")
        return NsDataObject(ns_provider_meta=kwargs,
                            ns_provider_name="POSTGRESQL",
                            ns_provider_type=self.type)

    def terminal(self, **kwargs):
        logger.info("Configuring Terminal provider")
        return NsDataObject(ns_provider_meta=kwargs,
                            ns_provider_name="TERMINAL",
                            ns_provider_type=self.type)

    def nsnode(self, **kwargs):
        logger.info("Configuring Node provider")
        return NsDataObject(ns_provider_meta=kwargs,
                            ns_provider_name="NODE",
                            ns_provider_type=self.type)


class Nstream(object):

    def __init__(self, provider: NsProvider) -> None:
        self.provider = provider
        self.event = "EVENT"
        logger.info("Nstream initialized")
        pass
    
    @staticmethod
    def event()->str:
        return "$EVENT"


class NsLink(Nstream):

    def __init__(self,
                 provider: NsProvider,
                 socket: NsSocket = None,
                 prompt_text: Optional[str] = None,
                 context_transform_prompt_text: Optional[str] = None) -> None:
        self.socket = socket
        self.provider = provider
        self.prompt_text = prompt_text
        self.context_prompt_text = context_transform_prompt_text
        logger.info(f"NsLink initialized with provider: {self.provider.NsProviderName}")
        return super().__init__(provider=self.provider)


    def define_prompt(self):
        logger.debug("Defining prompt for NsLink")
        return "{}: \n {}".format(self.prompt_text, self.event)

    def define_context(self):
        logger.debug("Defining context for NsLink")
        return "{}: \n {}".format(self.context_prompt_text, self.event)

    def process_sink(self, node_id) -> None:
        #  datasink - prompt
        avg_throughput = random.uniform(300, 500)
        provider_name = self.provider.NsProviderName
        link_metadata = {provider_name: "value"}
        data_input = self.provider.NsProviderType
        role = "output"
        prompt_mutation = create_data_detail_mutation(data_input, node_id,
                                                      avg_throughput,
                                                      link_metadata, role)
        _ = send_graphql_request(self.socket.dashboard_server,
                                 self.socket.headers, prompt_mutation)



class NsNode(object):

    def __init__(self,
                 node_name: str,
                 prompt: NsLink,
                 context: NsLink,
                 neuron: NsNeuron,
                 socket: NsSocket = None) -> None:
        self.node_name = node_name
        self.prompt = prompt
        self.context = context
        self.neuron = neuron
        self.socket = socket
        logger.info(f"NsNode initialized with name: {node_name}")

    def output(self, context_transform_prompt_text: Optional[str]):
        logger.info("Creating output for NsNode")
        out = NsLink(provider=NsProvider("SOURCE").nsnode(),
                     context_transform_prompt_text=context_transform_prompt_text)
        return out

    
    def process(self) -> None:
        prompt_size = random.randint(20, 80)
        context_size = random.randint(70, 120)
        total_data_processed = random.randint(5, 20)
        node_mutation = create_node_detail_mutation(self.node_name,
                                                    context_size, prompt_size,
                                                    self.prompt.prompt_text, self.context.context_prompt_text,
                                                    total_data_processed,
                                                    self.neuron.llm)
        response = send_graphql_request(self.socket.dashboard_server,
                                        self.socket.headers, node_mutation)
        self.node_id = json.loads(
            response.text).get("data").get("createNodeDetail").get("id")

        #  datasource - prompt
        avg_throughput = random.randint(300, 500)
        provider_name = self.prompt.provider.NsProviderName
        link_metadata = {provider_name: "value"}
        data_input = self.prompt.provider.NsProviderType
        role = "prompt"
        prompt_mutation = create_data_detail_mutation(data_input, self.node_id,
                                                      avg_throughput,
                                                      link_metadata, role)
        response = send_graphql_request(self.socket.dashboard_server,
                                        self.socket.headers, prompt_mutation)

        #  datasource - context
        avg_throughput = random.randint(300, 500)
        provider_name = self.context.provider.NsProviderName
        link_metadata = {provider_name: "value"}
        data_input = self.context.provider.NsProviderType
        role = "context"
        context_mutation = create_data_detail_mutation(data_input,
                                                       self.node_id,
                                                       avg_throughput,
                                                       link_metadata, role)
        response = send_graphql_request(self.socket.dashboard_server,
                                        self.socket.headers, context_mutation)

        return None



from setuptools import setup, find_packages
import sys
import pathlib

here = pathlib.Path(__file__).parent.resolve()

# Get the long description from the README file
long_description = (here / "README.md").read_text(encoding="utf-8")

extras = {
    'windows': ['pywin32>=300'],
    'linux': ['uwsgi>=2.0.18']
}

# Choose the right dependency
if sys.platform.startswith('win'):
    platform_specific_requires = extras['windows']
elif sys.platform.startswith('linux'):
    platform_specific_requires = extras['linux']
else:
    platform_specific_requires = []
# if sys.platform == 'darwin':  # macOS
#     extension_mod.extra_compile_args = ['-stdlib=libc++']

setup(
    name='nstreamai',
    version='0.0.11',
    description='Official SDK for nstream ai stream processing powered by Gen AI',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    keywords="nstreamai, streaming, rag, analytics, realtime",
    package_dir={"": "src"},
    python_requires=">=3.8, <4",
    packages=find_packages(where="src"),
    url='https://github.com/nstream-ai/ms-public-sdk',
    author='Nstream AI',
    author_email='hello@nstream.ai',
    license='MIT',
    install_requires=[
        "annotated-types==0.6.0",
        "anyio==4.3.0",
        "certifi==2024.2.2",
        "charset-normalizer==3.3.2",
        "distro==1.9.0",
        "exceptiongroup==1.2.0",
        "h11==0.14.0",
        "httpcore==1.0.5",
        "httpx==0.27.0",
        "idna==3.7",
        "openai==1.19.0",
        "pydantic==2.7.0",
        "pydantic_core==2.18.1",
        "pyfiglet==1.0.2",
        "requests==2.31.0",
        "sniffio==1.3.1",
        "termcolor==2.4.0",
        "tqdm==4.66.2",
        "typing_extensions==4.11.0",
        "urllib3==2.2.1"
    ]+platform_specific_requires,
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.org/classifiers/
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
    ],
)
